import thunk from 'redux-thunk';
import {createStore, combineReducers, applyMiddleware} from 'redux';
import { actionPending, actionFulfilled, actionRejected, actionPromise, actionRegister, actionLogin, actionAuthLogin, actionAuthLogout, actionUploadFile, actionUploadFiles, actionAdUpsert } from './actions';

const backendURL = 'http://marketplace.asmer.fs.a-level.com.ua';

//Шаблон запроса GQL
const getGQL = url =>
    (query, variables) => fetch(url, {
        method: 'POST',
        headers: {
            "Content-Type": "application/json",
            ...(localStorage.authToken ? {"Authorization": "Bearer " + localStorage.authToken} : {})
        },
        body: JSON.stringify({query, variables})
    }).then(res => res.json())
        .then(data => {
            if (data.data){
                return Object.values(data.data)[0] 
            } else {
                throw new Error(JSON.stringify(data.errors))
            }
        })

//Запрос на бэк marketplace 
const gql = getGQL(backendURL + '/graphql');


//Регистрация

//Авторизация
const jwtDecode = token => { 
    try{
        return JSON.parse(atob(token.split('.')[1]));
    }
    catch(e){
        return null;
    }
}


function promiseReducer(state = {}, { type, name, status, payload, error }) {
    if (type === "PROMISE") {
        return {
            ...state,
            [name]: { status, payload, error },
        };
    }
    return state;
}

    function authReducer(state, { type, token }) {
    if (state === undefined && localStorage.authToken) {
      token = localStorage.authToken
      type = 'AUTH_LOGIN'
    }
    if (type === 'AUTH_LOGIN') {
      if (jwtDecode(token)) {
        localStorage.authToken = token
        return { token, payload: jwtDecode(token) }
      }
    }
    if (type === 'AUTH_LOGOUT') {
      localStorage.authToken = ''
      return {}
    }
    return state || {}
  }

const store = createStore(combineReducers({
    promise: promiseReducer,
    auth: authReducer,
   }), applyMiddleware(thunk));

export {store, backendURL, gql};